﻿# -*- coding: utf-8 -*-

# File: PageBuilder.py
#
# Copyright (c) 2006 by []
# Generator: ArchGenXML Version 1.4.1
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Max M <maxm@mxm.dk>"""
__docformat__ = 'plaintext'

# cmf imports
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.PageBuilder.config import *

schema = Schema((

    StringField(
        name='title',
        widget=StringWidget(
            label='Title',
            label_msgid='PageBuilder_label_title',
            i18n_domain='PageBuilder',
        )
    ),

    TextField(
        name='description',
        widget=TextAreaWidget(
            label='Description',
            label_msgid='PageBuilder_label_description',
            i18n_domain='PageBuilder',
        )
    ),

    TextField('text',
        default_content_type='text/html',
        default_output_type='text/html',
        allowable_content_types=('text/plain', 'text/structured', 'text/html',),
        widget=RichWidget(
            label='Text',
            label_msgid='PageBuilder_label_text',
            description_msgid='PageBuilder_help_text',
            i18n_domain='PageBuilder',
        ),
    ),

),
)

PageBuilder_schema = OrderedBaseFolderSchema.copy() + \
    schema.copy()

class PageBuilder(OrderedBaseFolder):
    
    security = ClassSecurityInfo()
    
    __implements__ = (getattr(OrderedBaseFolder,'__implements__',()),)

    # This name appears in the 'add' box
    archetype_name = 'PageBuilder'

    meta_type = 'PageBuilder'
    portal_type = 'PageBuilder'
    allowed_content_types = ['PageRow']
    filter_content_types = 1
    global_allow = 1
    allow_discussion = False
    content_icon = 'PageBuilder.gif'
    immediate_view = 'pagebuilder_view'
    default_view = 'pagebuilder_view'
    suppl_views = ('pagebuilder_view',)
    typeDescription = "PageBuilder"
    typeDescMsgId = 'description_edit_pagebuilder'

    schema = PageBuilder_schema

    _at_rename_after_creation = True # rename object according to the title?

    actions = (
        {
            'id': 'view',
            'name': 'View',
            'action': 'string:${object_url}/pagebuilder_view',
            'permissions': (View,)
        },
        {
            'id': 'edit',
            'name': 'Edit',
            'action': 'string:${object_url}/pagebuilder_edit',
            'permissions': (ModifyPortalContent,)
        },
    )

    # Plone mechanics
    
    security.declarePrivate('at_post_create_script')
    def at_post_create_script(self):
        "Adds the first row automatically"
        self.addPageRow()
#        self.invokeFactory('PageRow', 'row_1')

    security.declarePublic('getAllowedTypes')
    def getAllowedTypes(self):
        "Returns list of allowed portal types"
        return []

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """
        Returns a concatination of all searchable text in the PageBuilder object.
        It can turn up false results, if a word exists in a column that isn't 
        visible due to the selected row macro.
        """
        txt = []
        txt.append(self.Title())
        txt.append(self.Description())
        txt.append(self.getText())
        for row in self.getPageRows():
            for col in row.getPageColumns():
                txt.append(col.Title())
                txt.append(col.Description())
                txt.append(col.getText())
        return ' '.join(txt)


    security.declareProtected(View, 'getPageBuilder')
    def getPageBuilder(self):
        "Returns the pagebuilder"
        return self


    security.declareProtected(View, 'pagebuilder_url')
    def pagebuilder_url(self):
        "Redirects to the absolute url of the pagebuilder"
        return self.absolute_url()

    security.declareProtected(ModifyPortalContent, 'addPageRow')
    def addPageRow(self, portal_type='PageRow', REQUEST=None):
        "Add a pagerow to the page"
        now = self.ZopeTime()
        id = '%s_%s' % (portal_type, now.strftime('%Y%m%d-%H%M%S'))
        self.invokeFactory(portal_type, id, title=portal_type)
        newRow = getattr(self, id)
        newRow.markCreationFlag()
        if hasattr(newRow, 'at_post_create_script'):
            newRow.at_post_create_script()
        if not REQUEST is None:
            REQUEST.RESPONSE.redirect(self.absolute_url() + '/pagebuilder_edit')


    security.declareProtected(ModifyPortalContent, 'movePageRow')
    def movePageRow(self, ids, delta=1, REQUEST=None):
        "Move a pagerow up or down the page"
        self.moveObjectsByDelta(ids, delta)
#            self.plone_utils.reindexOnReorder(self)
        if not REQUEST is None:
            REQUEST.RESPONSE.redirect(self.absolute_url() + '/pagebuilder_edit')


    security.declareProtected(ModifyPortalContent, 'delPageRow')
    def delPageRow(self, rowId, REQUEST):
        "Add a pagerow to the page"
        self.manage_delObjects(rowId)
        REQUEST.RESPONSE.redirect(self.absolute_url() + '/pagebuilder_edit')


    security.declareProtected(View, 'getPageRows')
    def getPageRows(self):
        "Returns the sorted list of page rows"
        # Filters on allowed types, for easy plugin in if needed
        return self.listFolderContents(
            contentFilter={'portal_type':self.getAddableTypes()}
            )

    security.declareProtected(View, 'getPageRows')
    def setPageRowMacro(self, rowId, macroname, REQUEST):
        "Sets the macroname of the row"
        row = getattr(self, rowId)
        row.setMacroName(macroname)
        REQUEST.RESPONSE.redirect(self.absolute_url() + '/pagebuilder_edit')


    security.declareProtected(View, 'setPageColMacro')
    def setPageColMacro(self, rowId, colId, macroname, REQUEST):
        "Sets the macroname of the row"
        row = getattr(self, rowId)
        col = getattr(row, colId)
        col.setMacroName(macroname)
        REQUEST.RESPONSE.redirect(self.absolute_url() + '/pagebuilder_edit')


    security.declareProtected(View, 'getMacroIcon')
    def getMacroIcon(self, obj, macroName):
        """
        Returns an icon for the row. icon name is generally same as macro name
        """
        iconPattern = '%s.gif'
        if obj.getMacroName() == macroName:
            iconPattern = iconPattern % '%s_active'
        iconId = iconPattern % macroName
        if not hasattr(self, iconId):
           iconId = iconPattern % 'PageBuilder_unknown'
        return getattr(self, iconId)

    security.declareProtected(ModifyPortalContent, 'getAddableTypes')
    def getAddableTypes(self):
        "Returns the portal types that can be added as rows."
        ptypes = getToolByName(self, 'portal_types')
        typeinfo = ptypes.getTypeInfo(self)
        return getattr(typeinfo, 'allowed_content_types', [])

    

registerType(PageBuilder,PROJECTNAME)
